import json

from messagebus import (
    AckMessage,
    Event,
    Message,
    MessageBus,
    NakMessage,
)
from messageprocessorutil import UuidMessageProcessor
from typing import Any

from services.interactor import DiscountAppliedInteractor
from services.model import DiscountAppliedRequest


class DiscountAppliedProcessor(UuidMessageProcessor):
    def __init__(
        self,
        discount_applied_interactor,    # type: DiscountAppliedInteractor
    ):
        # type: (...) -> None

        super(DiscountAppliedProcessor, self).__init__()
        self.discount_applied_interactor = discount_applied_interactor

    def parse_data(
        self,
        data,           # type: str
        event_type,     # type: str
    ):
        # type: (...) -> DiscountAppliedRequest

        request_json = json.loads(data)

        return DiscountAppliedRequest(
            pos_id=request_json.get("posId"),
            order_id=request_json.get("orderId"),
        )

    def call_business(
        self,
        request,  # type: DiscountAppliedRequest
    ):
        # type: (...) -> None

        self.discount_applied_interactor.process_request(request)

    def format_response(
        self,
        message_bus,    # type: MessageBus
        message,        # type: Message
        event,          # type: Event
        request,        # type: DiscountAppliedRequest
        result,         # type: None
    ):
        # type: (...) -> None

        message_bus.reply_event(
            event=event,
            reply_message=AckMessage(data=result),
        )

    def format_parse_exception(
        self,
        message_bus,    # type: MessageBus
        message,        # type: Message
        event,          # type: Event
        data,           # type: Any
        exception,      # type: Exception
    ):
        # type: (...) -> None

        return None

    def format_exception(
        self,
        message_bus,    # type: MessageBus
        message,        # type: Message
        event,          # type: Event
        input_obj,      # type: DiscountAppliedRequest
        exception,      # type: Exception
    ):
        # type: (...) -> None

        message_bus.reply_event(
            event=event,
            reply_message=NakMessage(data=exception.message),
        )



import logging

from logging import Logger
from typing import Optional

from cfgtools import read
from helper import config_logger

from services.model import Configs


class Configurator(object):

    def __init__(
        self,
        loader_path,            # type: str
        databases_path,         # type: str
        backup_databases_path,  # type: str
    ):
        # type: (...) -> None

        self.config = Configs()
        self.config.loader_path = loader_path
        self.config.databases_path = databases_path
        self.config.backup_databases_path = backup_databases_path
        self.reader = read(file_or_path=loader_path)
        self.group = None
        self.logger = None

    def read_loader_configs(
        self,
    ):
        # type: (...) -> Configs

        self.configure_loggers()
        self.config.component_name = self.reader.find_value("Identification.ComponentName")
        self.group = "Alerts"
        alerts_group = self.reader.find_group(self.group)
        alerts_config = {}
        for key in alerts_group.keys:
            alerts_config[key.name] = key.values[0] if len(key.values) == 1 else key.values
        self.config.alerts = alerts_config
        self.config.alerts[u"EndPoints"] = {
            u"ConfiguredAlerts": str(self._get_config("EndPoints.ConfiguredAlerts"))
        }

        return self.config

    def configure_loggers(
        self,
    ):
        # type: (...) -> None

        config_logger(
            loader_path=self.config.loader_path,
            log_name="Alerts",
            max_size=10485760,
            max_files=15,
        )
        self.logger = logging.getLogger(name="Alerts")

    def get_loggers(
        self,
    ):
        # type: (...) -> Logger

        return self.logger

    def _get_config(
        self,
        key,          # type: str
        array=False,  # type: Optional[bool]
    ):
        # type: (...) -> str

        path = self.config.component_name + "." + key
        return self.reader.find_values(path) if array else self.reader.find_value(path)

    @staticmethod
    def _is_true(
        value,  # type: str
    ):
        # type: (...) -> bool

        return value.lower() == "true"
